@@ -30,7 +30,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 47780 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 60935 $")
 
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
@@ -94,7 +94,7 @@ static void retrieve_file(char *dir)
 	int res;
 	int fd=-1;
 	size_t fdlen = 0;
-	void *fdm=NULL;
+	void *fdm = MAP_FAILED;
 	SQLHSTMT stmt;
 	char sql[256];
 	char fmt[80]="";
@@ -161,7 +161,7 @@ static void retrieve_file(char *dir)
 				if (fd > -1)
 					fdm = mmap(NULL, fdlen, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 			}
-			if (fdm) {
+			if (fdm != MAP_FAILED) {
 				memset(fdm, 0, fdlen);
 				res = SQLGetData(stmt, x + 1, SQL_BINARY, fdm, fdlen, &colsize);
 				if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
@@ -174,7 +174,7 @@ static void retrieve_file(char *dir)
 		} while (0);
 	} else
 		ast_log(LOG_WARNING, "Failed to obtain database object for '%s'!\n", odbc_database);
-	if (fdm)
+	if (fdm != MAP_FAILED)
 		munmap(fdm, fdlen);
 	if (fd > -1)
 		close(fd);
@@ -470,7 +470,7 @@ static int do_directory(struct ast_channel *chan, struct ast_config *cfg, char *
 							pos = strrchr(pos, ' ') + 1;
 						conv = convert(pos);
 						if (conv) {
-							if (!strcmp(conv, ext)) {
+							if (!strncmp(conv, ext, strlen(ext))) {
 								/* Match! */
 								found++;
 								free(conv);
