@@ -1,24 +1,29 @@
 /*
- * Asterisk -- A telephony toolkit for Linux.
+ * Asterisk -- An open source telephony toolkit.
  *
- * Use /dev/dsp as an intercom.
- * 
- * Copyright (C) 1999, Mark Spencer
+ * Copyright (C) 1999 - 2005, Digium, Inc.
  *
- * Mark Spencer <markster@linux-support.net>
+ * Mark Spencer <markster@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
  *
  * This program is free software, distributed under the terms of
- * the GNU General Public License
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Use /dev/dsp as an intercom.
+ * 
+ * \ingroup applications
  */
  
-#include <asterisk/lock.h>
-#include <asterisk/file.h>
-#include <asterisk/frame.h>
-#include <asterisk/logger.h>
-#include <asterisk/channel.h>
-#include <asterisk/pbx.h>
-#include <asterisk/module.h>
-#include <asterisk/translate.h>
+#include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
 #include <sys/ioctl.h>
@@ -35,6 +40,19 @@
 #include <soundcard.h>
 #endif
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
+
+#include "asterisk/lock.h"
+#include "asterisk/file.h"
+#include "asterisk/frame.h"
+#include "asterisk/logger.h"
+#include "asterisk/channel.h"
+#include "asterisk/pbx.h"
+#include "asterisk/module.h"
+#include "asterisk/translate.h"
+
 #ifdef __OpenBSD__
 #define DEV_DSP "/dev/audio"
 #else
@@ -51,8 +69,7 @@ static char *app = "Intercom";
 static char *synopsis = "(Obsolete) Send to Intercom";
 static char *descrip = 
 "  Intercom(): Sends the user to the intercom (i.e. /dev/dsp).  This program\n"
-"is generally considered  obselete by the chan_oss module.  Returns 0 if the\n"
-"user exits with a DTMF tone, or -1 if they hangup.\n";
+"is generally considered  obselete by the chan_oss module.  User can terminate\n"with a DTMF tone, or by hangup.\n";
 
 STANDARD_LOCAL_USER;
 
@@ -141,6 +158,7 @@ static int intercom_exec(struct ast_channel *chan, void *data)
 	res = ast_set_read_format(chan, AST_FORMAT_SLINEAR);
 	if (res < 0) {
 		ast_log(LOG_WARNING, "Unable to set format to signed linear on channel %s\n", chan->name);
+		LOCAL_USER_REMOVE(u);
 		return -1;
 	}
 	/* Read packets from the channel */
@@ -168,18 +186,27 @@ static int intercom_exec(struct ast_channel *chan, void *data)
 				res = -1;
 		}
 	}
-	LOCAL_USER_REMOVE(u);
+	
 	if (!res)
 		ast_set_read_format(chan, oreadformat);
+
+	LOCAL_USER_REMOVE(u);
+
 	return res;
 }
 
 int unload_module(void)
 {
-	STANDARD_HANGUP_LOCALUSERS;
+	int res;
+
 	if (sound > -1)
 		close(sound);
-	return ast_unregister_application(app);
+
+	res = ast_unregister_application(app);
+
+	STANDARD_HANGUP_LOCALUSERS;
+
+	return res;
 }
 
 int load_module(void)
