@@ -1,21 +1,28 @@
-/* codec_alaw.c - translate between signed linear and alaw
- * 
- * Asterisk -- A telephony toolkit for Linux.
+/*
+ * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (c) 2001 Linux Support Services, Inc.  All rights reserved.
+ * Copyright (C) 1999 - 2005, Digium, Inc.
  *
- * Mark Spencer <markster@linux-support.net
+ * Mark Spencer <markster@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
  *
  * This program is free software, distributed under the terms of
- * the GNU General Public License
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief codec_alaw.c - translate between signed linear and alaw
+ * 
+ * \ingroup codecs
  */
 
-#include <asterisk/lock.h>
-#include <asterisk/logger.h>
-#include <asterisk/module.h>
-#include <asterisk/translate.h>
-#include <asterisk/channel.h>
-#include <asterisk/alaw.h>
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <stdio.h>
@@ -23,6 +30,19 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
+
+#include "asterisk/lock.h"
+#include "asterisk/logger.h"
+#include "asterisk/module.h"
+#include "asterisk/config.h"
+#include "asterisk/options.h"
+#include "asterisk/translate.h"
+#include "asterisk/channel.h"
+#include "asterisk/alaw.h"
+
 #define BUFFER_SIZE   8096	/* size for the translation buffers */
 
 AST_MUTEX_DEFINE_STATIC(localuser_lock);
@@ -30,37 +50,38 @@ static int localusecnt = 0;
 
 static char *tdesc = "A-law Coder/Decoder";
 
+static int useplc = 0;
+
 /* Sample frame data (Mu data is okay) */
 
 #include "slin_ulaw_ex.h"
 #include "ulaw_slin_ex.h"
 
-/*
- * Private workspace for translating signed linear signals to alaw.
+/*!
+ * \brief Private workspace for translating signed linear signals to alaw.
  */
-
 struct alaw_encoder_pvt
 {
   struct ast_frame f;
-  char offset[AST_FRIENDLY_OFFSET];   /* Space to build offset */
-  unsigned char outbuf[BUFFER_SIZE];  /* Encoded alaw, two nibbles to a word */
+  char offset[AST_FRIENDLY_OFFSET];   /*!< Space to build offset */
+  unsigned char outbuf[BUFFER_SIZE];  /*!< Encoded alaw, two nibbles to a word */
   int tail;
 };
 
-/*
- * Private workspace for translating alaw signals to signed linear.
+/*!
+ * \brief Private workspace for translating alaw signals to signed linear.
  */
-
 struct alaw_decoder_pvt
 {
   struct ast_frame f;
   char offset[AST_FRIENDLY_OFFSET];	/* Space to build offset */
   short outbuf[BUFFER_SIZE];	/* Decoded signed linear values */
   int tail;
+  plc_state_t plc;
 };
 
-/*
- * alawToLin_New
+/*!
+ * \brief alawToLin_New
  *  Create a new instance of alaw_decoder_pvt.
  *
  * Results:
@@ -70,8 +91,7 @@ struct alaw_decoder_pvt
  *  None.
  */
 
-static struct ast_translator_pvt *
-alawtolin_new (void)
+static struct ast_translator_pvt * alawtolin_new (void)
 {
   struct alaw_decoder_pvt *tmp;
   tmp = malloc (sizeof (struct alaw_decoder_pvt));
@@ -79,14 +99,15 @@ alawtolin_new (void)
     {
 	  memset(tmp, 0, sizeof(*tmp));
       tmp->tail = 0;
+      plc_init(&tmp->plc);
       localusecnt++;
       ast_update_use_count ();
     }
   return (struct ast_translator_pvt *) tmp;
 }
 
-/*
- * LinToalaw_New
+/*!
+ * \brief LinToalaw_New
  *  Create a new instance of alaw_encoder_pvt.
  *
  * Results:
@@ -96,8 +117,7 @@ alawtolin_new (void)
  *  None.
  */
 
-static struct ast_translator_pvt *
-lintoalaw_new (void)
+static struct ast_translator_pvt * lintoalaw_new (void)
 {
   struct alaw_encoder_pvt *tmp;
   tmp = malloc (sizeof (struct alaw_encoder_pvt));
@@ -111,8 +131,8 @@ lintoalaw_new (void)
   return (struct ast_translator_pvt *) tmp;
 }
 
-/*
- * alawToLin_FrameIn
+/*!
+ * \brief alawToLin_FrameIn
  *  Fill an input buffer with packed 4-bit alaw values if there is room
  *  left.
  *
@@ -130,6 +150,18 @@ alawtolin_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
   int x;
   unsigned char *b;
 
+  if(f->datalen == 0) { /* perform PLC with nominal framesize of 20ms/160 samples */
+        if((tmp->tail + 160)  * 2 > sizeof(tmp->outbuf)) {
+            ast_log(LOG_WARNING, "Out of buffer space\n");
+            return -1;
+        }
+        if(useplc) {
+	    plc_fillin(&tmp->plc, tmp->outbuf+tmp->tail, 160);
+	    tmp->tail += 160;
+	}
+        return 0;
+  }
+
   if ((tmp->tail + f->datalen) * 2 > sizeof(tmp->outbuf)) {
   	ast_log(LOG_WARNING, "Out of buffer space\n");
 	return -1;
@@ -140,12 +172,14 @@ alawtolin_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
   for (x=0;x<f->datalen;x++)
   	tmp->outbuf[tmp->tail + x] = AST_ALAW(b[x]);
 
+  if(useplc) plc_rx(&tmp->plc, tmp->outbuf+tmp->tail, f->datalen);
+
   tmp->tail += f->datalen;
   return 0;
 }
 
-/*
- * alawToLin_FrameOut
+/*!
+ * \brief alawToLin_FrameOut
  *  Convert 4-bit alaw encoded signals to 16-bit signed linear.
  *
  * Results:
@@ -156,8 +190,7 @@ alawtolin_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
  *  None.
  */
 
-static struct ast_frame *
-alawtolin_frameout (struct ast_translator_pvt *pvt)
+static struct ast_frame * alawtolin_frameout (struct ast_translator_pvt *pvt)
 {
   struct alaw_decoder_pvt *tmp = (struct alaw_decoder_pvt *) pvt;
 
@@ -176,8 +209,8 @@ alawtolin_frameout (struct ast_translator_pvt *pvt)
   return &tmp->f;
 }
 
-/*
- * LinToalaw_FrameIn
+/*!
+ * \brief LinToalaw_FrameIn
  *  Fill an input buffer with 16-bit signed linear PCM values.
  *
  * Results:
@@ -187,8 +220,7 @@ alawtolin_frameout (struct ast_translator_pvt *pvt)
  *  tmp->tail is number of signal values in the input buffer.
  */
 
-static int
-lintoalaw_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
+static int lintoalaw_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
 {
   struct alaw_encoder_pvt *tmp = (struct alaw_encoder_pvt *) pvt;
   int x;
@@ -205,8 +237,8 @@ lintoalaw_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
   return 0;
 }
 
-/*
- * LinToalaw_FrameOut
+/*!
+ * \brief LinToalaw_FrameOut
  *  Convert a buffer of raw 16-bit signed linear PCM to a buffer
  *  of 4-bit alaw packed two to a byte (Big Endian).
  *
@@ -217,8 +249,7 @@ lintoalaw_framein (struct ast_translator_pvt *pvt, struct ast_frame *f)
  *  Leftover inbuf data gets packed, tail gets updated.
  */
 
-static struct ast_frame *
-lintoalaw_frameout (struct ast_translator_pvt *pvt)
+static struct ast_frame * lintoalaw_frameout (struct ast_translator_pvt *pvt)
 {
   struct alaw_encoder_pvt *tmp = (struct alaw_encoder_pvt *) pvt;
   
@@ -237,12 +268,11 @@ lintoalaw_frameout (struct ast_translator_pvt *pvt)
 }
 
 
-/*
- * alawToLin_Sample
+/*!
+ * \brief alawToLin_Sample
  */
 
-static struct ast_frame *
-alawtolin_sample (void)
+static struct ast_frame * alawtolin_sample (void)
 {
   static struct ast_frame f;
   f.frametype = AST_FRAME_VOICE;
@@ -256,12 +286,11 @@ alawtolin_sample (void)
   return &f;
 }
 
-/*
- * LinToalaw_Sample
+/*!
+ * \brief LinToalaw_Sample
  */
 
-static struct ast_frame *
-lintoalaw_sample (void)
+static struct ast_frame * lintoalaw_sample (void)
 {
   static struct ast_frame f;
   f.frametype = AST_FRAME_VOICE;
@@ -276,8 +305,8 @@ lintoalaw_sample (void)
   return &f;
 }
 
-/*
- * alaw_Destroy
+/*!
+ * \brief alaw_Destroy
  *  Destroys a private workspace.
  *
  * Results:
@@ -287,16 +316,15 @@ lintoalaw_sample (void)
  *  None.
  */
 
-static void
-alaw_destroy (struct ast_translator_pvt *pvt)
+static void alaw_destroy (struct ast_translator_pvt *pvt)
 {
   free (pvt);
   localusecnt--;
   ast_update_use_count ();
 }
 
-/*
- * The complete translator for alawToLin.
+/*!
+ * \brief The complete translator for alawToLin.
  */
 
 static struct ast_translator alawtolin = {
@@ -311,8 +339,8 @@ static struct ast_translator alawtolin = {
   alawtolin_sample
 };
 
-/*
- * The complete translator for LinToalaw.
+/*!
+ * \brief The complete translator for LinToalaw.
  */
 
 static struct ast_translator lintoalaw = {
@@ -327,8 +355,33 @@ static struct ast_translator lintoalaw = {
   lintoalaw_sample
 };
 
-int
-unload_module (void)
+static void parse_config(void)
+{
+  struct ast_config *cfg;
+  struct ast_variable *var;
+
+  if ((cfg = ast_config_load("codecs.conf"))) {
+    if ((var = ast_variable_browse(cfg, "plc"))) {
+      while (var) {
+       if (!strcasecmp(var->name, "genericplc")) {
+         useplc = ast_true(var->value) ? 1 : 0;
+         if (option_verbose > 2)
+           ast_verbose(VERBOSE_PREFIX_3 "codec_alaw: %susing generic PLC\n", useplc ? "" : "not ");
+       }
+       var = var->next;
+      }
+    }
+    ast_config_destroy(cfg);
+  }
+}
+
+int reload(void)
+{
+  parse_config();
+  return 0;
+}
+
+int unload_module (void)
 {
   int res;
   ast_mutex_lock (&localuser_lock);
@@ -341,10 +394,10 @@ unload_module (void)
   return res;
 }
 
-int
-load_module (void)
+int load_module (void)
 {
   int res;
+  parse_config();
   res = ast_register_translator (&alawtolin);
   if (!res)
     res = ast_register_translator (&lintoalaw);
@@ -357,22 +410,19 @@ load_module (void)
  * Return a description of this module.
  */
 
-char *
-description (void)
+char * description (void)
 {
   return tdesc;
 }
 
-int
-usecount (void)
+int usecount (void)
 {
   int res;
   STANDARD_USECOUNT (res);
   return res;
 }
 
-char *
-key ()
+char * key ()
 {
   return ASTERISK_GPL_KEY;
 }
