@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-/* ASTERISK_FILE_VERSION(__FILE__, "$Revision: 43924 $") */
+/* ASTERISK_FILE_VERSION(__FILE__, "$Revision: 61680 $") */
 
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -42,6 +42,9 @@ static char *builtin_function_timeout_read(struct ast_channel *chan, char *cmd,
 {
 	time_t myt;
 
+	if (!chan)
+		return "";
+
 	if (!data) {
 		ast_log(LOG_ERROR, "Must specify type of timeout to get.\n");
                 return NULL;
@@ -86,6 +89,9 @@ static void builtin_function_timeout_write(struct ast_channel *chan, char *cmd,
 	char timestr[64];
 	struct tm myt;
 
+	if (!chan)
+		return;
+
 	if (!data) {
 		ast_log(LOG_ERROR, "Must specify type of timeout to set.\n");
 		return;
