@@ -21,6 +21,8 @@
 #define DEFAULT_LANGUAGE "en"
 
 #define AST_CONFIG_MAX_PATH 255
+#define	setpriority	__PLEASE_USE_ast_set_priority_INSTEAD_OF_setpriority__
+#define	sched_setscheduler	__PLEASE_USE_ast_set_priority_INSTEAD_OF_sched_setscheduler__
 
 /* provided in asterisk.c */
 extern char ast_config_AST_CONFIG_DIR[AST_CONFIG_MAX_PATH];
