@@ -1,5 +1,15 @@
 /*
- * BSD Telephony Of Mexico "Tormenta" Tone Zone Support 2/22/01
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ */
+
+/*! \file
+ * \brief BSD Telephony Of Mexico "Tormenta" Tone Zone Support 2/22/01
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +32,7 @@
 #ifndef _ASTERISK_INDICATIONS_H
 #define _ASTERISK_INDICATIONS_H
 
-#include <asterisk/lock.h>
+#include "asterisk/lock.h"
 
 /* forward reference */
 struct ast_channel;
@@ -43,8 +53,8 @@ struct tone_zone {
 	char country[5];				/* Country code */
 	char alias[5];					/* is this an alias? */
 	char description[40];				/* Description */
-	int  nrringcadance;				/* # registered ringcadance elements */
-	int *ringcadance;				/* Ring cadence */
+	int  nrringcadence;				/* # registered ringcadence elements */
+	int *ringcadence;				/* Ring cadence */
 	struct tone_zone_sound *tones;			/* The known tones for this zone */
 };
 
@@ -74,4 +84,4 @@ void ast_playtones_stop(struct ast_channel *chan);
 extern struct tone_zone *tone_zones;
 extern ast_mutex_t tzlock;
 
-#endif
+#endif /* _ASTERISK_INDICATIONS_H */
