@@ -20,7 +20,11 @@
 #define HAVE_STRERROR	1
 #define	HAVE_UNISTD_H	1
 #define	LOCALE_HOME	_PATH_LOCALE
+#ifdef SOLARIS
+#define TZDIR		"/usr/share/lib/zoneinfo"
+#else
 #define TZDIR		"/usr/share/zoneinfo"
+#endif /* def SOLARIS */
 #endif /* ndef TM_GMTOFF */
 
 /*
